<?php
require_once("vmlib/logUtil.inc");

Class FmeServer {
    public $sUrl;
    public $sTimeUnit;
    public $iExpiration;
    public $aLastCurlRequestInfo;
    private $sUser;
    private $sPassword;
    private $sToken;
    
    /**
     * construct
     * @param {string} $sUrl Url of the fme server.
     * @param {string} $sUser User id.
     * @param {string} $sPassword User password.
     * @param {string} $sTimeUnit Time unit (day, hour, minute, second).
     * @param {number} $iExpiration Token expiration.
     * @param {boolean} $bUpdate New (true) or current token if exists (false).
     */
    function __construct ($sUrl, $sUser, $sPassword, $sTimeUnit = 'day', $iExpiration = 1, $bUpdate = false) {
        $this->sUrl = $sUrl;
        $this->sUser = $sUser;
        $this->sPassword = $sPassword;
        $this->sTimeUnit = $sTimeUnit;  // Unité de temps pour la durée du token (day, hour, minute, second).
        $this->iExpiration = $iExpiration;  // Durée de validité du token.
        $this->bUpdate = $bUpdate;  // true -> génère un nouveau token écrase l’ancien / false -> retourne le token courant.
    }

    /**
     * Get a FME Server Token
     * @return Token
     */
    function getToken() {
        // Url vers le serveur fme.
        $sUrl = $this->sUrl;
        // Url vers le service de génération d'un token.
        $sUrl = $this->sUrl . '/fmetoken/service/generate';
        // Données de la requête.
        $aRequestData = array (
            'user' => $this->sUser,
            'password' => $this->sPassword,
            'expiration' => $this->iExpiration,
            'timeunit' => $this->sTimeUnit,
            'update' => $this->bUpdate
        );
        // Transfert cURL
        $this->sToken = $this->curlRequest($sUrl, 'post', $aRequestData);
        return $this->sToken;
    }
    
    /**
     * Curl request
     * @param {string} $sUrl Url of the Curl request.
     * @param {string} $sType Type of the Curl request (get, post).
     * @param {array} $aData Data of the Curl request.
     * @param {array} $aHeaders Headers of the Curl request.
     * @return Request result
     */
    // 
    function curlRequest($sUrl, $sType, $aData = array(), $aHeaders = array()) {
        //
        $this->aLastCurlRequestInfo = '';
        //
        $ch = curl_init();
        $sType = strtoupper($sType);
        // Force la méthode de requête utilisée (GET, POST...).
        curl_setopt($ch, CURLOPT_CUSTOMREQUEST, $sType);
        // Url à utiliser.
        if (($sType == 'GET' || $sType == 'DELETE') && !empty($aData))
            $sUrl .= '?' . http_build_query($aData);
        curl_setopt($ch, CURLOPT_URL, $sUrl);
        // Retour sous forme de texte. 
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
        // Requête POST.
        if ($sType == 'POST') {
            curl_setopt($ch, CURLOPT_POST, true);
            // Chaîne de requête en encodage URL.
            if (is_array($aData))
                $aData = http_build_query($aData);
            // Données de la requête.
            curl_setopt($ch, CURLOPT_POSTFIELDS, $aData);
            //
            curl_setopt($ch, CURLOPT_SAFE_UPLOAD, true);
            // Entête pour la requête en POST.
            //$aHeaders[] = 'Content-Type: application/x-www-form-urlencoded';
        }
        // Entête pour la requête
        $aHeaders[] = 'Accept: application/json';
        if (!empty($this->sToken))
            $aHeaders[] = 'Authorization: fmetoken token=' . $this->sToken; // Token obligatoire pour éxécuter la requête.
        curl_setopt($ch, CURLOPT_HTTPHEADER, $aHeaders);
        // Durée max. de la requête.
        //curl_setopt($ch, CURLOPT_TIMEOUT, 120);
        // Exécute la session CURL.
        // Log.
        /*
        $handle = fopen("curl_log.txt", "a");
        fwrite($handle, PHP_EOL . '-----------------------' . PHP_EOL);
        curl_setopt($ch, CURLOPT_VERBOSE, true);
        curl_setopt($ch, CURLOPT_STDERR , $handle);
        */
        curl_setopt($ch, CURLINFO_HEADER_OUT, true);
        // Curl error: SSL certificate problem: unable to get local issuer certificate
        // Curl error n°60
        curl_setopt ($ch, CURLOPT_SSL_VERIFYPEER, false);      
        //
        $output = curl_exec($ch);
        // Erreur de la requête CURL.
        if(curl_errno($ch)) {
            writeToErrorLog('Curl error: ' . curl_error($ch));
            writeToErrorLog('Curl error n°' . curl_errno($ch));
        }
        // Informations de la requête.
        $aCurlInfo = curl_getinfo($ch);
        $this->aLastCurlRequestInfo = $aCurlInfo;
        //file_put_contents('C:/svn/gtf_cloud/gtf.engines/log/response.log', print_r($aCurlInfo, true));
        //writeToErrorLog(print_r(curl_getinfo($ch), true));
        
        // Ferme la session CURL.
        curl_close($ch);
        //
        //fclose($handle);
        //
        return $output;
    }
    
    /**
     * Send a request to the REST API.
     * @param {string} $sService Name of the service.
     * @param {string} $sMethod Method of the service (get, post...).
     * @param {array} $aData Data of the Curl request.
     * @return Request result
     */
    function serviceRequest($sService, $sMethod = 'get', $aData = array()) {
        // Demande de token.
        $this->getToken();
        // Url vers le serveur fme.
        $sUrl = $this->sUrl;
        // Url vers le service d'information.
        $sUrl = $this->sUrl . '/fmerest/v3/' . $sService;
        // Transfert cURL
        return json_decode($this->curlRequest($sUrl, $sMethod, $aData));
    }
    
    /**
     * Upload a file
     * @param {string} $sFilePath Path of the file to upload.
     * @param {string} $sServiceUrl Url of the web service to upload the file.
     * @return Request result
     */
    function uploadFile($sServiceUrl, $sFilePath) {
        if (file_exists($sFilePath)) {
            // Demande de token.
            $this->getToken();
            // Nom du fichier.
            $sFileName = pathinfo($sFilePath, PATHINFO_BASENAME);
            // Type MIME du fichier.
            //$finfo = finfo_open(FILEINFO_MIME_TYPE);            
            // Crée un objet CURLFile.
            //$oCfile = new CURLFile($sFilePath, finfo_file($finfo, $sFilePath), $sFileName);
            //finfo_close($finfo);
            // Entêtes de la requête Curl.
            $aRequestHeaders = array(
                'Content-Type: application/octet-stream',
                'Content-Disposition: attachment; filename="' . $sFileName . '"',
                //'Content-Length: ' . filesize($sFilePath)
            );
            //
            return json_decode($this->curlRequest($sServiceUrl, 'post', file_get_contents($sFilePath), $aRequestHeaders));
        }
    }
    
    /**
     * Upload a file to the resource connection
     * @param {string} $sFilePath Path of the file to upload.
     * @param {string} $sResource Name of a resource connection.
     * @param {string} $sPath Path, relative to the resource connection.
     * @param {boolean} $bCreateDirectories Create directories corresponding to the file path.
     * @param {boolean} $bOverwrite Overwrite the file if it already exists.
     * @return Request result
     */
    function uploadResourceFile($sFilePath, $sResource, $sPath, $bCreateDirectories = false, $bOverwrite = false) {
        // Demande de token.
        $this->getToken();
        // Url du service de ressources.
        $sUrl = $this->sUrl . '/fmerest/v3/resources/connections/' . $sResource . '/filesys/' . $sPath;
        $sUrl .= '?createDirectories=' . var_export($bCreateDirectories, true) . '&overwrite=' . var_export($bOverwrite, true);
        return $this->uploadFile($sUrl, $sFilePath);
    }
    
    /**
     * Upload a file to the resource connection
     * @param {string} $sFilePath Path of the file to upload.
     * @param {string} $sRepository Name of the repository.
     * @param {boolean} $bOverwrite Overwrite the file if it already exists.
     * @return Request result
     */
    function uploadWorkspaceFile($sFilePath, $sRepository, $bOverwrite = false) {
        // Demande de token.
        $this->getToken();
        // Supprime le fichier s'il existe déja.
        if ($bOverwrite) {
            $sFileName = pathinfo($sFilePath, PATHINFO_BASENAME);
            $this->serviceRequest('repositories/' . $sRepository . '/items/' . $sFileName);
            if ($this->aLastCurlRequestInfo['http_code'] == 200)
                $this->serviceRequest('repositories/' . $sRepository . '/items/' . $sFileName, 'delete');
        }
        // Url du service vers le dépôt spécifié.
        $sUrl = $this->sUrl . '/fmerest/v3/repositories/' . $sRepository . '/items';
        return $this->uploadFile($sUrl, $sFilePath);
    }
    
    /**
     * Get information about a repository item.
     * @param {string} $sRepository Name of the repository.
     * @param {string} $sWorkspace Name of the repository item.
     * @return Request result
     */
    function getWorkspaceParams($sRepository, $sWorkspace) {
        // Demande de token.
        $this->getToken();
        // Url du service vers le traitement (fichier.fmw) spécifié.
        $sUrl = $this->sUrl . '/fmerest/v3/repositories/' . $sRepository . '/items/' . $sWorkspace;
        // Transfert cURL.
        return json_decode($this->curlRequest($sUrl, 'get'));
    }
    
    /**
     * Submit a job to run a transformation.
     * @param {string} $sRepository Name of the repository.
     * @param {string} $sWorkspace Name of the repository item.
     * @param {array} $aParams The definition of the transformation request.
     * @param {boolean} $bAsync Asynchronous job.
     * @return Request result
     */
    function submitJob($sRepository, $sWorkspace, $aParams, $bAsync = false) {
        // Demande de token.
        $this->getToken();
        // Url du service vers le traitement (fichier.fmw) spécifié.
        if ($bAsync)
            $sUrl = $this->sUrl . '/fmerest/v3/transformations/submit/' . $sRepository . '/' . $sWorkspace;
        else
            $sUrl = $this->sUrl . '/fmerest/v3/transformations/transact/' . $sRepository . '/' . $sWorkspace;
        // Données au format json.
        $aHeaders = array('Content-Type: application/json');
        // Transfert cURL.
        return json_decode($this->curlRequest($sUrl, 'post', json_encode($aParams), $aHeaders));
    }
    
    /**
     * Downloads a file from a resource connection.
     * @param {string} $sResource Name of a resource connection.
     * @param {string} $sPath Path, relative to the resource connection.
     * @param {string} $sFilePath Path of the file to save.
     * @return File content
     */
    function downloadResourceFile($sResource, $sPath, $sFilePath = null) {
        // Demande de token.
        $this->getToken();
        // Url du service vers le traitement (fichier.fmw) spécifié.
        $sUrl = $this->sUrl . '/fmerest/v3/resources/connections/' . $sResource . '/filesys/' . $sPath;
        // Obligatoire pour récupérer le contenu du fichier.
        $aHeaders = array('Accept: application/octet-stream');
        // Données de la requête.
        /*
        $aRequestData = array (
            'accept' => 'contents',
            'disposition' => 'inline'
        );
        */
        // Transfert cURL.
        $sFileContent = $this->curlRequest($sUrl, 'get', null, $aHeaders);
        if (!empty($sFilePath))
            file_put_contents($sFilePath, $sFileContent);
        else
            return $sFileContent;
    }
    
    /**
     * Get the log file of a job.
     * @param {number} $sJobId Job id.
     * @return Log file content
     */
    function getJobLog($sJobId, $sFilePath = '') {
        // Demande de token.
        $this->getToken();
        // Url du service vers le traitement (fichier.fmw) spécifié.
        $sUrl = $this->sUrl . '/fmerest/v3/transformations/jobs/id/' . $sJobId . '/log';
        // Obligatoire pour récupérer le contenu du fichier.
        $aHeaders = array('Accept: text/plain');
        // Transfert cURL.
        $sFileContent = $this->curlRequest($sUrl, 'get', null, $aHeaders);
        if (!empty($sFilePath))
            file_put_contents($sFilePath, $sFileContent);
        else
            return $sFileContent;
    }
    
    /**
     * Create a repository to the FME Server instance.
     * @param {string} $sRepository Name of the repository.
     * @param {string} $sDescription Description of the repository.
     * @return Request result
     */
    function createRepository($sRepository, $sDescription) {
        // Demande de token.
        $this->getToken();
        // Crée le dépot s'il n'existe pas déja.
        $aRequestResult = $this->serviceRequest('repositories/' . $sRepository);
        if ($this->aLastCurlRequestInfo['http_code'] == 404) {
            $aRequestData = array (
                'name' => $sRepository,
                'description' => $sDescription
            );
            $aRequestResult = $this->serviceRequest('repositories', 'post', $aRequestData);
        }
        return $aRequestResult;
    }
    
    /**
     * Upload a file to the resource connection from Amazon S3 service.
     * @param {string} $aS3Params Parameters of the Amazon S3 service.
     * @param {string} $sRepository Name of the repository.
     * @param {string} $sResource Name of a resource connection.
     * @param {string} $sPath Path, relative to the resource connection.
     * @return Request result
     */
    function uploadResourceFileFromS3($aS3Params, $sRepository, $sResource, $sPath) {
        // Demande de token.
        $this->getToken();
        //
        $aJobParams = array(
            'publishedParameters' => array(
                array('name' => 'access_key_id', 'value' => $aS3Params['access_key_id']),
                array('name' => 'secret_access_key', 'value' => $aS3Params['secret_access_key']),
                array('name' => 'bucket_name', 'value' => $aS3Params['bucket_name']),
                array('name' => 'object_key', 'value' => $aS3Params['object_key']),
                array('name' => 'target_file', 'value' => '$(' . $sResource . ')/' . $sPath . '/' . pathinfo($aS3Params['object_key'], PATHINFO_BASENAME))
            )
        );
        $oJobResult = $this->submitJob($sRepository, 'S3Downloader.fmw', $aJobParams);
        return $oJobResult;
    }
    
    /**
     * Create a repository to the FME Server instance.
     * @param {string} $sResource Name of a resource connection.
     * @param {string} $sPath Path, relative to the resource connection.
     * @param {string} $DirectoryName Name of the directory.
     * @return Request result
     */
    function createResourceConnectionDirectory($sResource, $sPath, $DirectoryName) {
        // Demande de token.
        $this->getToken();
        // Crée le répertoire s'il n'existe pas déja.
        $aRequestResult = $this->serviceRequest('resources/connections/' . $sResource . '/filesys/' . $sPath . '/' . $DirectoryName);
        if ($this->aLastCurlRequestInfo['http_code'] == 404) {
            $aRequestData = array (
                'directoryname' => $DirectoryName
            );
            $aRequestResult = $this->serviceRequest('resources/connections/' . $sResource . '/filesys/' . $sPath, 'post', $aRequestData);
        }
        return $aRequestResult;
    }
    
    /**
     * Get the FME Server build number and version.
     * @return Request result
     */
    function getInfo() {
        // Demande de token.
        $this->getToken();
        // Url du web service info.
        $sUrl = $this->sUrl . '/fmerest/v3/info';
        // Transfert cURL.
        return json_decode($this->curlRequest($sUrl, 'get'));
    }
    
    /**
     * Get status of the installed FME Server license.
     * @return Request result
     */
    function getLicenceStatus() {
        // Demande de token.
        $this->getToken();
        // Url du web service info.
        $sUrl = $this->sUrl . '/fmerest/v3/licensing/license/status';
        // Transfert cURL.
        return json_decode($this->curlRequest($sUrl, 'get'));
    }
}
?>