<?php

/**
 * \file FmeEngines.class.inc
 * \class FmeEngines
 *
 * \author Yoann Perollet <yoann.perollet@veremes.com>.
 *
 * 	\brief This file contains the Modes php class
 *
 * This class defines Rest Api to Vitis FmeEngines
 * 
 */
require_once 'Gtf.class.inc';
require_once __DIR__ . '/../../class/vitis_lib/Connection.class.inc';
require_once 'FmeEngine.class.inc';
require_once __DIR__ . '/../../class/vmlib/BdDataAccess.inc';

class FmeEngines extends GTF {
    /**
     * @SWG\Definition(
     *   definition="/fmeengines",
     *   allOf={
     *     @SWG\Schema(ref="#/definitions/fmeengines")
     *   }
     * )
     * * @SWG\Tag(
     *   name="FmeEngines",
     *   description="Operations about Fme Engines"
     * )
     */

    /**
     * construct
     * @param type $aPath url of the request
     * @param type $aValues parameters of the request
     * @param type $properties properties
     * @param type $bShortcut false to reinit variables
     * @param type $oConnection connection object
     */
    function __construct($aPath, $aValues, $properties, $bShortcut = false, $oConnection = false) {
        parent::__construct($aPath, $aValues, $properties, $bShortcut, $oConnection);
        $this->aSelectedFields = Array("fme_engine_id", "name", "server_id", "local_path", "server_name", "fme_engine_type_id", "server_url", "login", "password", "s3_access_key_id", "s3_secret_access_key", "fme_cloud_api_token", "fme_engine_type_label", "s3_region", "iam_access_key_id", "iam_secret_access_key");
    }

    /**
     * @SWG\Get(path="/fmeengines",
     *   tags={"FmeEngines"},
     *   summary="Get Fme Engines",
     *   description="Request to get Fme Engines",
     *   operationId="GET",
     *   produces={"application/xml", "application/json", "application/x-vm-json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="fmeengine token",
     *     required=true,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="order_by",
     *     in="query",
     *     description="list of ordering fields",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="sort_order",
     *     in="query",
     *     description="sort order",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="limit",
     *     in="query",
     *     description="number of element",
     *     required=false,
     *     type="integer",
     *     format="int32"
     *   ),
     * @SWG\Parameter(
     *     name="offset",
     *     in="query",
     *     description="index of first element",
     *     required=false,
     *     type="string",
     *   ),
     * @SWG\Parameter(
     *     name="attributs",
     *     in="query",
     *     description="list of attributs",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="filter",
     *     in="query",
     *     description="filter results",
     *     required=false,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="distinct",
     *     in="query",
     *     description="delete duplicates",
     *     required=false,
     *     type="boolean"
     *   ),
     *   @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/fmeengines")
     *     )
     *  )
     */
    /**
     * @SWG\Get(path="/fmeengines/{fme_engine_id}/Test", 
     *   tags={"FmeEngines"},
     *   summary="Get FmeEngine test",
     *   description="Request to get FmeEnginetest by id",
     *   operationId="GET",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="user token",
     *     required=true,
     *     type="string"
     *   ),
     * @SWG\Parameter(
     *     name="fme_engine_id",
     *     in="path",
     *     description="user id",
     *     required=true,
     *     type="integer",
     *   format="int32"
     *   ),
     * @SWG\Parameter(
     *     name="attributs",
     *     in="query",
     *     description="list of attributs",
     *     required=false,
     *     type="string"
     *   ),
     *   @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/fmeengines")
     *     )
     *  )
     */

    /**
     * get FmeEngines
     * @return  FmeEngines
     */
    function GET() {
        $aReturn = $this->genericGet($this->aProperties['schema_gtf'], "v_fme_engine", "fme_engine_id");
        if (!empty($this->aPath[3]) && $this->aPath[3] == 'Test') {
            if ($aReturn['sStatus'] == 1) {
                // Test de validité de la licence FME.
                $this->testFmeLicence();
                $aXmlRacineAttribute['status'] = 1;
                $sMessage = $this->asDocument('', 'vitis', $this->aValues['sEncoding'], True, $aXmlRacineAttribute, $this->aValues['sSourceEncoding'], $this->aValues['output']);
                return $sMessage;
            }
        } else
            return $aReturn['sMessage'];
    }

    /**
     * @SWG\Post(path="/fmeengines",
     *   tags={"FmeEngines"},
     *   summary="Add FmeEngine",
     *   description="Request to add fmeengine",
     *   operationId="POST",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="formData",
     *     description="fmeengine token",
     *     required=true,
     *     type="string"
     *   ),
     *   @SWG\Parameter(
     *     name="name",
     *     in="formData",
     *     description="Name of the fme engine",
     *     required=true,
     *     type="string"
     *   ),
     *   @SWG\Parameter(
     *     name="server_id",
     *     in="formData",
     *     description="Server of fme engine",
     *     required=true,
     *     type="integer",
     *     format="int32"
     *   ),
     *   @SWG\Parameter(
     *     name="local_path",
     *     in="formData",
     *     description="local_path of the fme engine",
     *     required=true,
     *     type="string"
     *   ),
     *   @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/fmeengines")
     *     )
     *  )
     */

    /**
     * insert fmeengine
     * @return id of the fmeengine created
     */
    function POST() {
        $aReturn = $this->genericPost($this->aProperties['schema_gtf'], 'v_fme_engine', $this->aProperties['schema_gtf'] . '.seq_common', 'fme_engine_id');
        if ($aReturn['sStatus'] == 1) {
            $aXmlRacineAttribute['status'] = 1;
            $sMessage = $this->asDocument('', 'vitis', $this->aValues['sEncoding'], True, $aXmlRacineAttribute, $this->aValues['sSourceEncoding'], $this->aValues['output']);
            $oFmeEngine = new FmeEngine($this->aPath, $this->aValues, $this->aProperties, $this->oConnection);
            $oFmeEngine->GET();
        } else {
            $sMessage = $aReturn['sMessage'];
        }
        return $sMessage;
    }

    /**
     * @SWG\Put(path="/fmeengines/{fme_engine_id}",
     *   tags={"FmeEngines"},
     *   summary="Update FmeEngine",
     *   description="Request to update FmeEngine",
     *   operationId="PUT",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="fmeengine token",
     *     required=true,
     *     type="string"
     *   ),
     * * @SWG\Parameter(
     *     name="fme_engine_id",
     *     in="path",
     *     description="id of the fme engine",
     *     required=true,
     *     type="integer",
     *     format = "int32"
     *   ),
     *   @SWG\Parameter(
     *     name="name",
     *     in="query",
     *     description="Name of the fme engine",
     *     required=true,
     *     type="string"
     *   ),
     *   @SWG\Parameter(
     *     name="server_id",
     *     in="query",
     *     description="Server of fme engine",
     *     required=true,
     *     type="integer",
     *     format="int32"
     *   ),
     *   @SWG\Parameter(
     *     name="local_path",
     *     in="query",
     *     description="local_path of the fme engine",
     *     required=true,
     *     type="string"
     *   ),
     *   @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/fmeengines")
     *     )
     *  )
     */

    /**
     * update fmeengine
     * @return id of the fmeengine if ok error object if ko
     */
    function PUT() {
        $aReturn = $this->genericPut($this->aProperties['schema_gtf'], 'v_fme_engine', 'fme_engine_id');
        if ($aReturn['sStatus'] == 1) {
            $aXmlRacineAttribute['status'] = 1;
            $sMessage = $this->asDocument('', 'vitis', $this->aValues['sEncoding'], True, $aXmlRacineAttribute, $this->aValues['sSourceEncoding'], $this->aValues['output']);
            $oFmeEngine = new FmeEngine($this->aPath, $this->aValues, $this->aProperties, $this->oConnection);
            $oFmeEngine->GET();
        } else {
            $sMessage = $aReturn['sMessage'];
        }
        return $sMessage;
    }

    /**
     * @SWG\Delete(path="/fmeengines/",
     *   tags={"FmeEngines"},
     *   summary="delete FmeEngine",
     *   description="Request to delete FmeEngine",
     *   operationId="DELETE",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="fmeengine token",
     *     required=true,
     *     type="string"
     *   ),
     * * @SWG\Parameter(
     *     name="idList",
     *     in="query",
     *     description="id of the fmeengines",
     *     required=true,
     *     type="string"
     *   ),
     * @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/fmeengines")
     *     )
     *  )
     */
    /**
     * @SWG\Delete(path="/fmeengines/{fme_engine_id}",
     *   tags={"FmeEngines"},
     *   summary="delete FmeEngine",
     *   description="Request to delete FmeEngine",
     *   operationId="DELETE",
     *   produces={"application/xml", "application/json"},
     *   @SWG\Parameter(
     *     name="token",
     *     in="query",
     *     description="fmeengine token",
     *     required=true,
     *     type="string"
     *   ),
     * * @SWG\Parameter(
     *     name="fme_engine_id",
     *     in="path",
     *     description="id of the fmeengine",
     *     required=true,
     *     type="integer",
     *     format = "int32"
     *   ),
     * @SWG\Response(
     *         response=200,
     *         description="Poprerties Response",
     *         @SWG\Schema(ref="#/definitions/fmeengines")
     *     )
     *  )
     */

    /**
     * delete fmeengine.
     * @return id of fmeengine deleted or error object if a fmeengine is not deleted
     */
    function DELETE() {
        $aReturn = $this->genericDelete($this->aProperties['schema_gtf'], 'v_fme_engine', 'fme_engine_id');
        return $aReturn['sMessage'];
    }

    /**
     * get fme licence parameters.
     * @return id of fmeengine deleted or error object if a fmeengine is not deleted
     */
    function getFmeLicence($fmepath) {
        // Si le fichier existe et la fonction exec ne renvoie pas d'erreur : remplissage du tableau
        if (file_exists($fmepath)) {
            // Le fichier existe
            $this->aObjects[0]->aFields['exists'] = true;

            // Exécute fme.exe
            $aOutput = array();
            $iReturnStatus = 1;
            
            // Variable d'environement "SystemDrive" obligatoire sinon erreur avec FME !
            if (strtolower(substr(PHP_OS, 0, 3)) == 'win')
                putenv('SystemDrive=' . substr($fmepath, 0, 2));
            
            // Paramètre "INFO" obligatoire pour FME 2016 sinon pas de ligne "Temporary License".
            exec('"' . $fmepath . '"' . ' INFO', $aOutput, $iReturnStatus);
            // Paramètre "INFO" non reconnu par FME 2014 (erreur).
            if ($iReturnStatus === 1)
                exec($fmepath, $aOutput, $iReturnStatus);

            // Le fichier fme.exe s'est exécuté normalement ?
            if ($iReturnStatus === 0) {
                // Est ce que la licence a expiré ?
                if (stripos(implode($aOutput), 'licence has expired') !== false)
                    $this->aObjects[0]->aFields['valid'] = false;
                else {
                    // La licence est valide	
                    $this->aObjects[0]->aFields['valid'] = true;

                    // Version du logiciel
                    $this->aObjects[0]->aFields['version'] = trim($aOutput[0]);

                    // Type de licence
                    $this->aObjects[0]->aFields['type'] = trim($aOutput[1]);

                    // Validité de la licence
                    $this->aObjects[0]->aFields['validity'] = trim($aOutput[2]);

                    // Host
                    $this->aObjects[0]->aFields['host'] = trim(substr($aOutput[3], stripos($aOutput[3], ':') + 1));
                }
            }
        }
    }
    
    /**
     * Test Fme licence.
     */
    function testFmeLicence() {
        // Tableau de retour contenant les infos de la licence
        $aFmeLicence = array('exists' => false, 'valid' => false, 'version' => '', 'type' => '', 'validity' => '', 'host' => '');
        $this->aObjects[0]->aFields = array_merge($this->aObjects[0]->aFields, $aFmeLicence);
        //
        if (empty($this->aObjects[0]->aFields['fme_engine_type_id']))
            $this->aObjects[0]->aFields['fme_engine_type_id'] = 'desktop';
        switch ($this->aObjects[0]->aFields['fme_engine_type_id']) {
            case 'desktop':
                $this->getFmeLicence($this->aObjects[0]->aFields["local_path"]);
            break;
            case 'fme_server':
                require 'gtf_lib/FmeServer.class.inc';
                $oFmeServer = new FmeServer($this->aObjects[0]->aFields['server_url'], $this->aObjects[0]->aFields['login'], $this->aObjects[0]->aFields['password'], 'day', 1);
                $oFmeServerLicenceStatus = $oFmeServer->getLicenceStatus();
                $this->aObjects[0]->aFields['exists'] = true;
                if ($oFmeServerLicenceStatus->isLicenseExpired === true)
                    $this->aObjects[0]->aFields['expired'] = true;
                else if ($oFmeServerLicenceStatus->isLicensed === false)
                    $this->aObjects[0]->aFields['valid'] = false;
                else {
                    // Info sur la licence de FME Server.
                    //$this->aObjects[0]->aFields['exists'] = true;
                    $this->aObjects[0]->aFields['valid'] = true;
                    $sExpiryDate = $oFmeServerLicenceStatus->expiryDate;
                    if (!empty($sExpiryDate))
                        $sExpiryDate = substr($sExpiryDate, 6, 2) . '/' . substr($sExpiryDate, 4, 2) . '/' . substr($sExpiryDate, 0, 4);
                    $this->aObjects[0]->aFields['validity'] = $sExpiryDate;
                    // Info sur FME Server.
                    $oFmeServerInfo = $oFmeServer->getInfo();
                    $this->aObjects[0]->aFields['version'] = $oFmeServerInfo->build;
                    $this->aObjects[0]->aFields['type'] = $oFmeServerInfo->version;
                    $this->aObjects[0]->aFields['host'] = parse_url($this->aObjects[0]->aFields['server_url'], PHP_URL_HOST);
                }
            break;
            case 'fme_cloud':
                error_log(print_r($this->aObjects[0]->aFields, true));
            break;
        }
    }
}
?>